/* Copyright (c) 2002 Jeff Johnston  <jjohnstn@redhat.com> */
#ifndef	_MACHSTDLIB_H_
#define	_MACHSTDLIB_H_

/* place holder so platforms may add stdlib.h extensions */

#endif	/* _MACHSTDLIB_H_ */


