// GENERATED FILE - DO NOT EDIT
// This file was generated using generate-stubs.py with --target=x86_64-apple-macos10.12

.global __Exit
__Exit:

.global ___bzero
___bzero:

.global ___darwin_check_fd_set
___darwin_check_fd_set:

.global ___darwin_check_fd_set_overflow
___darwin_check_fd_set_overflow:

.global ___darwin_fd_clr
___darwin_fd_clr:

.global ___darwin_fd_isset
___darwin_fd_isset:

.global ___darwin_fd_set
___darwin_fd_set:

.global ___error
___error:

.global ___fpclassify
___fpclassify:

.global ___fpclassifyd
___fpclassifyd:

.global ___fpclassifyf
___fpclassifyf:

.global ___inf
___inf:

.global ___inff
___inff:

.global ___infl
___infl:

.global ___math_errhandling
___math_errhandling:

.global ___mb_cur_max
___mb_cur_max:

.global ___nan
___nan:

.global ___opendir2$INODE64
___opendir2$INODE64:

.global ___snprintf_chk
___snprintf_chk:

.global ___sprintf_chk
___sprintf_chk:

.global ___sputc
___sputc:

.global ___srget
___srget:

.global ___stderrp
___stderrp:

.global ___stdinp
___stdinp:

.global ___stdoutp
___stdoutp:

.global ___svfscanf
___svfscanf:

.global ___swbuf
___swbuf:

.global ___vsnprintf_chk
___vsnprintf_chk:

.global ___vsprintf_chk
___vsprintf_chk:

.global __exit
__exit:

.global _a64l
_a64l:

.global _abort
_abort:

.global _abs
_abs:

.global _access
_access:

.global _accessx_np
_accessx_np:

.global _acct
_acct:

.global _acos
_acos:

.global _acosf
_acosf:

.global _acosh
_acosh:

.global _acoshf
_acoshf:

.global _acoshl
_acoshl:

.global _acosl
_acosl:

.global _add_profil
_add_profil:

.global _alarm
_alarm:

.global _aligned_alloc
_aligned_alloc:

.global _alloca
_alloca:

.global _alphasort$INODE64
_alphasort$INODE64:

.global _arc4random
_arc4random:

.global _arc4random_addrandom
_arc4random_addrandom:

.global _arc4random_buf
_arc4random_buf:

.global _arc4random_stir
_arc4random_stir:

.global _arc4random_uniform
_arc4random_uniform:

.global _asctime
_asctime:

.global _asctime_r
_asctime_r:

.global _asin
_asin:

.global _asinf
_asinf:

.global _asinh
_asinh:

.global _asinhf
_asinhf:

.global _asinhl
_asinhl:

.global _asinl
_asinl:

.global _asprintf
_asprintf:

.global _atan
_atan:

.global _atan2
_atan2:

.global _atan2f
_atan2f:

.global _atan2l
_atan2l:

.global _atanf
_atanf:

.global _atanh
_atanh:

.global _atanhf
_atanhf:

.global _atanhl
_atanhl:

.global _atanl
_atanl:

.global _atexit
_atexit:

.global _atexit_b
_atexit_b:

.global _atof
_atof:

.global _atoi
_atoi:

.global _atol
_atol:

.global _atoll
_atoll:

.global _bcmp
_bcmp:

.global _bcopy
_bcopy:

.global _brk
_brk:

.global _bsearch
_bsearch:

.global _bsearch_b
_bsearch_b:

.global _bzero
_bzero:

.global _calloc
_calloc:

.global _cbrt
_cbrt:

.global _cbrtf
_cbrtf:

.global _cbrtl
_cbrtl:

.global _ceil
_ceil:

.global _ceilf
_ceilf:

.global _ceill
_ceill:

.global _cgetcap
_cgetcap:

.global _cgetclose
_cgetclose:

.global _cgetent
_cgetent:

.global _cgetfirst
_cgetfirst:

.global _cgetmatch
_cgetmatch:

.global _cgetnext
_cgetnext:

.global _cgetnum
_cgetnum:

.global _cgetset
_cgetset:

.global _cgetstr
_cgetstr:

.global _cgetustr
_cgetustr:

.global _chdir
_chdir:

.global _chflags
_chflags:

.global _chmod
_chmod:

.global _chmodx_np
_chmodx_np:

.global _chown
_chown:

.global _chroot
_chroot:

.global _clearerr
_clearerr:

.global _clock
_clock:

.global _clock_getres
_clock_getres:

.global _clock_gettime
_clock_gettime:

.global _clock_gettime_nsec_np
_clock_gettime_nsec_np:

.global _clock_settime
_clock_settime:

.global _close
_close:

.global _closedir
_closedir:

.global _confstr
_confstr:

.global _copysign
_copysign:

.global _copysignf
_copysignf:

.global _copysignl
_copysignl:

.global _cos
_cos:

.global _cosf
_cosf:

.global _cosh
_cosh:

.global _coshf
_coshf:

.global _coshl
_coshl:

.global _cosl
_cosl:

.global _creat
_creat:

.global _crypt
_crypt:

.global _ctermid
_ctermid:

.global _ctermid_r
_ctermid_r:

.global _ctime
_ctime:

.global _ctime_r
_ctime_r:

.global _daemon$1050
_daemon$1050:

.global _daylight
_daylight:

.global _devname
_devname:

.global _devname_r
_devname_r:

.global _difftime
_difftime:

.global _dirfd
_dirfd:

.global _div
_div:

.global _dprintf
_dprintf:

.global _drand48
_drand48:

.global _drem
_drem:

.global _dup
_dup:

.global _dup2
_dup2:

.global _ecvt
_ecvt:

.global _encrypt
_encrypt:

.global _endusershell
_endusershell:

.global _environ
_environ:

.global _erand48
_erand48:

.global _erf
_erf:

.global _erfc
_erfc:

.global _erfcf
_erfcf:

.global _erfcl
_erfcl:

.global _erff
_erff:

.global _erfl
_erfl:

.global _exchangedata
_exchangedata:

.global _execl
_execl:

.global _execle
_execle:

.global _execlp
_execlp:

.global _execv
_execv:

.global _execvP
_execvP:

.global _execve
_execve:

.global _execvp
_execvp:

.global _exit
_exit:

.global _exp
_exp:

.global _exp2
_exp2:

.global _exp2f
_exp2f:

.global _exp2l
_exp2l:

.global _expf
_expf:

.global _expl
_expl:

.global _expm1
_expm1:

.global _expm1f
_expm1f:

.global _expm1l
_expm1l:

.global _fabs
_fabs:

.global _fabsf
_fabsf:

.global _fabsl
_fabsl:

.global _faccessat
_faccessat:

.global _fchdir
_fchdir:

.global _fchflags
_fchflags:

.global _fchmod
_fchmod:

.global _fchmodat
_fchmodat:

.global _fchmodx_np
_fchmodx_np:

.global _fchown
_fchown:

.global _fchownat
_fchownat:

.global _fclose
_fclose:

.global _fcntl
_fcntl:

.global _fcvt
_fcvt:

.global _fdim
_fdim:

.global _fdimf
_fdimf:

.global _fdiml
_fdiml:

.global _fdopen
_fdopen:

.global _fdopendir$INODE64
_fdopendir$INODE64:

.global _feof
_feof:

.global _ferror
_ferror:

.global _fflagstostr
_fflagstostr:

.global _fflush
_fflush:

.global _ffs
_ffs:

.global _ffsctl
_ffsctl:

.global _ffsl
_ffsl:

.global _ffsll
_ffsll:

.global _fgetattrlist
_fgetattrlist:

.global _fgetc
_fgetc:

.global _fgetln
_fgetln:

.global _fgetpos
_fgetpos:

.global _fgets
_fgets:

.global _fileno
_fileno:

.global _filesec_dup
_filesec_dup:

.global _filesec_free
_filesec_free:

.global _filesec_get_property
_filesec_get_property:

.global _filesec_init
_filesec_init:

.global _filesec_query_property
_filesec_query_property:

.global _filesec_set_property
_filesec_set_property:

.global _filesec_unset_property
_filesec_unset_property:

.global _finite
_finite:

.global _flock
_flock:

.global _flockfile
_flockfile:

.global _floor
_floor:

.global _floorf
_floorf:

.global _floorl
_floorl:

.global _fls
_fls:

.global _flsl
_flsl:

.global _flsll
_flsll:

.global _fma
_fma:

.global _fmaf
_fmaf:

.global _fmal
_fmal:

.global _fmax
_fmax:

.global _fmaxf
_fmaxf:

.global _fmaxl
_fmaxl:

.global _fmemopen
_fmemopen:

.global _fmin
_fmin:

.global _fminf
_fminf:

.global _fminl
_fminl:

.global _fmod
_fmod:

.global _fmodf
_fmodf:

.global _fmodl
_fmodl:

.global _fmtcheck
_fmtcheck:

.global _fopen
_fopen:

.global _fork
_fork:

.global _fpathconf
_fpathconf:

.global _fprintf
_fprintf:

.global _fpurge
_fpurge:

.global _fputc
_fputc:

.global _fputs
_fputs:

.global _fread
_fread:

.global _free
_free:

.global _freopen
_freopen:

.global _frexp
_frexp:

.global _frexpf
_frexpf:

.global _frexpl
_frexpl:

.global _fscanf
_fscanf:

.global _fsctl
_fsctl:

.global _fseek
_fseek:

.global _fseeko
_fseeko:

.global _fsetattrlist
_fsetattrlist:

.global _fsetpos
_fsetpos:

.global _fstat$INODE64
_fstat$INODE64:

.global _fstat64
_fstat64:

.global _fstatat$INODE64
_fstatat$INODE64:

.global _fstatx64_np
_fstatx64_np:

.global _fstatx_np$INODE64
_fstatx_np$INODE64:

.global _fsync
_fsync:

.global _fsync_volume_np
_fsync_volume_np:

.global _ftell
_ftell:

.global _ftello
_ftello:

.global _ftruncate
_ftruncate:

.global _ftrylockfile
_ftrylockfile:

.global _funlockfile
_funlockfile:

.global _funopen
_funopen:

.global _futimens
_futimens:

.global _fwrite
_fwrite:

.global _gamma
_gamma:

.global _gcvt
_gcvt:

.global _getattrlist
_getattrlist:

.global _getattrlistat
_getattrlistat:

.global _getattrlistbulk
_getattrlistbulk:

.global _getbsize
_getbsize:

.global _getc
_getc:

.global _getc_unlocked
_getc_unlocked:

.global _getchar
_getchar:

.global _getchar_unlocked
_getchar_unlocked:

.global _getcwd
_getcwd:

.global _getdate
_getdate:

.global _getdate_err
_getdate_err:

.global _getdelim
_getdelim:

.global _getdirentries_is_not_available_when_64_bit_inodes_are_in_effect
_getdirentries_is_not_available_when_64_bit_inodes_are_in_effect:

.global _getdirentriesattr
_getdirentriesattr:

.global _getdomainname
_getdomainname:

.global _getdtablesize
_getdtablesize:

.global _getegid
_getegid:

.global _getenv
_getenv:

.global _geteuid
_geteuid:

.global _getgid
_getgid:

.global _getgrouplist
_getgrouplist:

.global _getgroups
_getgroups:

.global _gethostid
_gethostid:

.global _gethostname
_gethostname:

.global _gethostuuid
_gethostuuid:

.global _getiopolicy_np
_getiopolicy_np:

.global _getline
_getline:

.global _getloadavg
_getloadavg:

.global _getlogin
_getlogin:

.global _getlogin_r
_getlogin_r:

.global _getmode
_getmode:

.global _getopt
_getopt:

.global _getpagesize
_getpagesize:

.global _getpass
_getpass:

.global _getpeereid
_getpeereid:

.global _getpgid
_getpgid:

.global _getpgrp
_getpgrp:

.global _getpid
_getpid:

.global _getppid
_getppid:

.global _getpriority
_getpriority:

.global _getprogname
_getprogname:

.global _getrlimit
_getrlimit:

.global _getrusage
_getrusage:

.global _gets
_gets:

.global _getsgroups_np
_getsgroups_np:

.global _getsid
_getsid:

.global _getsubopt
_getsubopt:

.global _getuid
_getuid:

.global _getusershell
_getusershell:

.global _getw
_getw:

.global _getwd
_getwd:

.global _getwgroups_np
_getwgroups_np:

.global _gmtime
_gmtime:

.global _gmtime_r
_gmtime_r:

.global _grantpt
_grantpt:

.global _heapsort
_heapsort:

.global _heapsort_b
_heapsort_b:

.global _hypot
_hypot:

.global _hypotf
_hypotf:

.global _hypotl
_hypotl:

.global _ilogb
_ilogb:

.global _ilogbf
_ilogbf:

.global _ilogbl
_ilogbl:

.global _index
_index:

.global _initgroups
_initgroups:

.global _initstate
_initstate:

.global _iruserok
_iruserok:

.global _iruserok_sa
_iruserok_sa:

.global _isatty
_isatty:

.global _issetugid
_issetugid:

.global _j0
_j0:

.global _j1
_j1:

.global _jn
_jn:

.global _jrand48
_jrand48:

.global _l64a
_l64a:

.global _labs
_labs:

.global _lchflags
_lchflags:

.global _lchmod
_lchmod:

.global _lchown
_lchown:

.global _lcong48
_lcong48:

.global _ldexp
_ldexp:

.global _ldexpf
_ldexpf:

.global _ldexpl
_ldexpl:

.global _ldiv
_ldiv:

.global _lgamma
_lgamma:

.global _lgammaf
_lgammaf:

.global _lgammal
_lgammal:

.global _link
_link:

.global _linkat
_linkat:

.global _llabs
_llabs:

.global _lldiv
_lldiv:

.global _llrint
_llrint:

.global _llrintf
_llrintf:

.global _llrintl
_llrintl:

.global _llround
_llround:

.global _llroundf
_llroundf:

.global _llroundl
_llroundl:

.global _localtime
_localtime:

.global _localtime_r
_localtime_r:

.global _lockf
_lockf:

.global _log
_log:

.global _log10
_log10:

.global _log10f
_log10f:

.global _log10l
_log10l:

.global _log1p
_log1p:

.global _log1pf
_log1pf:

.global _log1pl
_log1pl:

.global _log2
_log2:

.global _log2f
_log2f:

.global _log2l
_log2l:

.global _logb
_logb:

.global _logbf
_logbf:

.global _logbl
_logbl:

.global _logf
_logf:

.global _logl
_logl:

.global _lrand48
_lrand48:

.global _lrint
_lrint:

.global _lrintf
_lrintf:

.global _lrintl
_lrintl:

.global _lround
_lround:

.global _lroundf
_lroundf:

.global _lroundl
_lroundl:

.global _lseek
_lseek:

.global _lstat$INODE64
_lstat$INODE64:

.global _lstat64
_lstat64:

.global _lstatx64_np
_lstatx64_np:

.global _lstatx_np$INODE64
_lstatx_np$INODE64:

.global _madvise
_madvise:

.global _malloc
_malloc:

.global _matherr
_matherr:

.global _mblen
_mblen:

.global _mbstowcs
_mbstowcs:

.global _mbtowc
_mbtowc:

.global _memccpy
_memccpy:

.global _memchr
_memchr:

.global _memcmp
_memcmp:

.global _memcpy
_memcpy:

.global _memmem
_memmem:

.global _memmove
_memmove:

.global _memset
_memset:

.global _memset_pattern16
_memset_pattern16:

.global _memset_pattern4
_memset_pattern4:

.global _memset_pattern8
_memset_pattern8:

.global _memset_s
_memset_s:

.global _mergesort
_mergesort:

.global _mergesort_b
_mergesort_b:

.global _mincore
_mincore:

.global _minherit
_minherit:

.global _mkdir
_mkdir:

.global _mkdirat
_mkdirat:

.global _mkdirx_np
_mkdirx_np:

.global _mkdtemp
_mkdtemp:

.global _mkdtempat_np
_mkdtempat_np:

.global _mkfifo
_mkfifo:

.global _mkfifox_np
_mkfifox_np:

.global _mknod
_mknod:

.global _mkostemp
_mkostemp:

.global _mkostemps
_mkostemps:

.global _mkostempsat_np
_mkostempsat_np:

.global _mkpath_np
_mkpath_np:

.global _mkpathat_np
_mkpathat_np:

.global _mkstemp
_mkstemp:

.global _mkstemp_dprotected_np
_mkstemp_dprotected_np:

.global _mkstemps
_mkstemps:

.global _mkstempsat_np
_mkstempsat_np:

.global _mktemp
_mktemp:

.global _mktime
_mktime:

.global _mlock
_mlock:

.global _mlockall
_mlockall:

.global _mmap
_mmap:

.global _modf
_modf:

.global _modff
_modff:

.global _modfl
_modfl:

.global _mprotect
_mprotect:

.global _mrand48
_mrand48:

.global _msync
_msync:

.global _munlock
_munlock:

.global _munlockall
_munlockall:

.global _munmap
_munmap:

.global _nan
_nan:

.global _nanf
_nanf:

.global _nanl
_nanl:

.global _nanosleep
_nanosleep:

.global _nearbyint
_nearbyint:

.global _nearbyintf
_nearbyintf:

.global _nearbyintl
_nearbyintl:

.global _nextafter
_nextafter:

.global _nextafterf
_nextafterf:

.global _nextafterl
_nextafterl:

.global _nexttoward
_nexttoward:

.global _nexttowardf
_nexttowardf:

.global _nexttowardl
_nexttowardl:

.global _nfssvc
_nfssvc:

.global _nice
_nice:

.global _nrand48
_nrand48:

.global _open
_open:

.global _open_dprotected_np
_open_dprotected_np:

.global _open_memstream
_open_memstream:

.global _openat
_openat:

.global _opendir$INODE64
_opendir$INODE64:

.global _openx_np
_openx_np:

.global _optarg
_optarg:

.global _opterr
_opterr:

.global _optind
_optind:

.global _optopt
_optopt:

.global _optreset
_optreset:

.global _pathconf
_pathconf:

.global _pause
_pause:

.global _pclose
_pclose:

.global _perror
_perror:

.global _pipe
_pipe:

.global _popen
_popen:

.global _posix2time
_posix2time:

.global _posix_madvise
_posix_madvise:

.global _posix_memalign
_posix_memalign:

.global _posix_openpt
_posix_openpt:

.global _pow
_pow:

.global _powf
_powf:

.global _powl
_powl:

.global _pread
_pread:

.global _printf
_printf:

.global _profil
_profil:

.global _pselect$1050
_pselect$1050:

.global _psort
_psort:

.global _psort_b
_psort_b:

.global _psort_r
_psort_r:

.global _pthread_getugid_np
_pthread_getugid_np:

.global _pthread_setugid_np
_pthread_setugid_np:

.global _ptsname
_ptsname:

.global _ptsname_r
_ptsname_r:

.global _putc
_putc:

.global _putc_unlocked
_putc_unlocked:

.global _putchar
_putchar:

.global _putchar_unlocked
_putchar_unlocked:

.global _putenv
_putenv:

.global _puts
_puts:

.global _putw
_putw:

.global _pwrite
_pwrite:

.global _qsort
_qsort:

.global _qsort_b
_qsort_b:

.global _qsort_r
_qsort_r:

.global _radixsort
_radixsort:

.global _rand
_rand:

.global _rand_r
_rand_r:

.global _random
_random:

.global _rcmd
_rcmd:

.global _rcmd_af
_rcmd_af:

.global _read
_read:

.global _readdir$INODE64
_readdir$INODE64:

.global _readdir_r$INODE64
_readdir_r$INODE64:

.global _readlink
_readlink:

.global _readlinkat
_readlinkat:

.global _realloc
_realloc:

.global _reallocf
_reallocf:

.global _realpath$DARWIN_EXTSN
_realpath$DARWIN_EXTSN:

.global _reboot
_reboot:

.global _remainder
_remainder:

.global _remainderf
_remainderf:

.global _remainderl
_remainderl:

.global _remove
_remove:

.global _remquo
_remquo:

.global _remquof
_remquof:

.global _remquol
_remquol:

.global _rename
_rename:

.global _renameat
_renameat:

.global _renameatx_np
_renameatx_np:

.global _renamex_np
_renamex_np:

.global _revoke
_revoke:

.global _rewind
_rewind:

.global _rewinddir$INODE64
_rewinddir$INODE64:

.global _rindex
_rindex:

.global _rint
_rint:

.global _rintf
_rintf:

.global _rintl
_rintl:

.global _rinttol
_rinttol:

.global _rmdir
_rmdir:

.global _round
_round:

.global _roundf
_roundf:

.global _roundl
_roundl:

.global _roundtol
_roundtol:

.global _rpmatch
_rpmatch:

.global _rresvport
_rresvport:

.global _rresvport_af
_rresvport_af:

.global _ruserok
_ruserok:

.global _sbrk
_sbrk:

.global _scalb
_scalb:

.global _scalbln
_scalbln:

.global _scalblnf
_scalblnf:

.global _scalblnl
_scalblnl:

.global _scalbn
_scalbn:

.global _scalbnf
_scalbnf:

.global _scalbnl
_scalbnl:

.global _scandir$INODE64
_scandir$INODE64:

.global _scandir_b$INODE64
_scandir_b$INODE64:

.global _scanf
_scanf:

.global _searchfs
_searchfs:

.global _seed48
_seed48:

.global _seekdir$INODE64
_seekdir$INODE64:

.global _select$1050
_select$1050:

.global _setattrlist
_setattrlist:

.global _setattrlistat
_setattrlistat:

.global _setbuf
_setbuf:

.global _setbuffer
_setbuffer:

.global _setdomainname
_setdomainname:

.global _setegid
_setegid:

.global _setenv
_setenv:

.global _seteuid
_seteuid:

.global _setgid
_setgid:

.global _setgroups
_setgroups:

.global _sethostid
_sethostid:

.global _sethostname
_sethostname:

.global _setiopolicy_np
_setiopolicy_np:

.global _setkey
_setkey:

.global _setlinebuf
_setlinebuf:

.global _setlogin
_setlogin:

.global _setmode
_setmode:

.global _setpgid
_setpgid:

.global _setpgrp
_setpgrp:

.global _setpriority
_setpriority:

.global _setprogname
_setprogname:

.global _setregid
_setregid:

.global _setreuid
_setreuid:

.global _setrgid
_setrgid:

.global _setrlimit
_setrlimit:

.global _setruid
_setruid:

.global _setsgroups_np
_setsgroups_np:

.global _setsid
_setsid:

.global _setstate
_setstate:

.global _setuid
_setuid:

.global _setusershell
_setusershell:

.global _setvbuf
_setvbuf:

.global _setwgroups_np
_setwgroups_np:

.global _shm_open
_shm_open:

.global _shm_unlink
_shm_unlink:

.global _signal
_signal:

.global _signgam
_signgam:

.global _significand
_significand:

.global _sin
_sin:

.global _sinf
_sinf:

.global _sinh
_sinh:

.global _sinhf
_sinhf:

.global _sinhl
_sinhl:

.global _sinl
_sinl:

.global _sleep
_sleep:

.global _snprintf
_snprintf:

.global _sprintf
_sprintf:

.global _sqrt
_sqrt:

.global _sqrtf
_sqrtf:

.global _sqrtl
_sqrtl:

.global _sradixsort
_sradixsort:

.global _srand
_srand:

.global _srand48
_srand48:

.global _sranddev
_sranddev:

.global _srandom
_srandom:

.global _srandomdev
_srandomdev:

.global _sscanf
_sscanf:

.global _stat$INODE64
_stat$INODE64:

.global _stat64
_stat64:

.global _statx64_np
_statx64_np:

.global _statx_np$INODE64
_statx_np$INODE64:

.global _stpcpy
_stpcpy:

.global _stpncpy
_stpncpy:

.global _strcasecmp
_strcasecmp:

.global _strcasestr
_strcasestr:

.global _strcat
_strcat:

.global _strchr
_strchr:

.global _strcmp
_strcmp:

.global _strcoll
_strcoll:

.global _strcpy
_strcpy:

.global _strcspn
_strcspn:

.global _strdup
_strdup:

.global _strerror
_strerror:

.global _strerror_r
_strerror_r:

.global _strftime
_strftime:

.global _strlcat
_strlcat:

.global _strlcpy
_strlcpy:

.global _strlen
_strlen:

.global _strmode
_strmode:

.global _strncasecmp
_strncasecmp:

.global _strncat
_strncat:

.global _strncmp
_strncmp:

.global _strncpy
_strncpy:

.global _strndup
_strndup:

.global _strnlen
_strnlen:

.global _strnstr
_strnstr:

.global _strpbrk
_strpbrk:

.global _strptime
_strptime:

.global _strrchr
_strrchr:

.global _strsep
_strsep:

.global _strsignal
_strsignal:

.global _strsignal_r
_strsignal_r:

.global _strspn
_strspn:

.global _strstr
_strstr:

.global _strtod
_strtod:

.global _strtof
_strtof:

.global _strtofflags
_strtofflags:

.global _strtok
_strtok:

.global _strtok_r
_strtok_r:

.global _strtol
_strtol:

.global _strtold
_strtold:

.global _strtoll
_strtoll:

.global _strtonum
_strtonum:

.global _strtoq
_strtoq:

.global _strtoul
_strtoul:

.global _strtoull
_strtoull:

.global _strtouq
_strtouq:

.global _strxfrm
_strxfrm:

.global _suboptarg
_suboptarg:

.global _swab
_swab:

.global _swapon
_swapon:

.global _symlink
_symlink:

.global _symlinkat
_symlinkat:

.global _sync
_sync:

.global _sync_volume_np
_sync_volume_np:

.global _sys_errlist
_sys_errlist:

.global _sys_nerr
_sys_nerr:

.global _syscall
_syscall:

.global _sysconf
_sysconf:

.global _system
_system:

.global _tan
_tan:

.global _tanf
_tanf:

.global _tanh
_tanh:

.global _tanhf
_tanhf:

.global _tanhl
_tanhl:

.global _tanl
_tanl:

.global _tcgetpgrp
_tcgetpgrp:

.global _tcsetpgrp
_tcsetpgrp:

.global _telldir$INODE64
_telldir$INODE64:

.global _tempnam
_tempnam:

.global _tgamma
_tgamma:

.global _tgammaf
_tgammaf:

.global _tgammal
_tgammal:

.global _time
_time:

.global _time2posix
_time2posix:

.global _timegm
_timegm:

.global _timelocal
_timelocal:

.global _timespec_get
_timespec_get:

.global _timezone
_timezone:

.global _timingsafe_bcmp
_timingsafe_bcmp:

.global _tmpfile
_tmpfile:

.global _tmpnam
_tmpnam:

.global _trunc
_trunc:

.global _truncate
_truncate:

.global _truncf
_truncf:

.global _truncl
_truncl:

.global _ttyname
_ttyname:

.global _ttyname_r
_ttyname_r:

.global _ttyslot
_ttyslot:

.global _tzname
_tzname:

.global _tzset
_tzset:

.global _tzsetwall
_tzsetwall:

.global _ualarm
_ualarm:

.global _umask
_umask:

.global _umaskx_np
_umaskx_np:

.global _undelete
_undelete:

.global _ungetc
_ungetc:

.global _unlink
_unlink:

.global _unlinkat
_unlinkat:

.global _unlockpt
_unlockpt:

.global _unsetenv
_unsetenv:

.global _unwhiteout
_unwhiteout:

.global _usleep
_usleep:

.global _utimensat
_utimensat:

.global _valloc
_valloc:

.global _vasprintf
_vasprintf:

.global _vdprintf
_vdprintf:

.global _vfork
_vfork:

.global _vfprintf
_vfprintf:

.global _vfscanf
_vfscanf:

.global _vprintf
_vprintf:

.global _vscanf
_vscanf:

.global _vsnprintf
_vsnprintf:

.global _vsprintf
_vsprintf:

.global _vsscanf
_vsscanf:

.global _wait
_wait:

.global _wait3
_wait3:

.global _wait4
_wait4:

.global _waitid
_waitid:

.global _waitpid
_waitpid:

.global _wcstombs
_wcstombs:

.global _wctomb
_wctomb:

.global _write
_write:

.global _y0
_y0:

.global _y1
_y1:

.global _yn
_yn:

.global _zopen
_zopen:

.global dyld_stub_binder
dyld_stub_binder:
