// Copyright (c) 2016-2017 Antony Polukhin
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_PFR_PRECISE_HPP
#define BOOST_PFR_PRECISE_HPP

/// \file boost/pfr/precise.hpp
/// Includes all the Boost.PFR headers that do not define `flat_*` functions, except \xmlonly<link linkend='header.boost.pfr.precise.global_ops_hpp'>boost/pfr/precise/global_ops.hpp</link>\endxmlonly

#include <boost/pfr/precise/core.hpp>
#include <boost/pfr/precise/functors.hpp>
#include <boost/pfr/precise/ops.hpp>
#include <boost/pfr/precise/io.hpp>
#include <boost/pfr/precise/tuple_size.hpp>
#include <boost/pfr/precise/functions_for.hpp>

#endif // BOOST_PFR_PRECISE_HPP
