// Include all header files for libSystem.dylib, so all the stubs are
// generated. Add more headers as needed.

#include <dirent.h>
#include <fcntl.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <sys/errno.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

// These symbols must be declared manually.
extern char **environ;
