// Hand created file. DO NOT DELETE.
// Hardfault aliases for definitions that have inconsistent naming (which are
// auto-generated by gen-device-svd.go) among devices in package nxp.

//go:build nxp && mimxrt1062
// +build nxp,mimxrt1062

package nxp

const (
	HardFault_CFSR_IACCVIOL    = SCB_CFSR_IACCVIOL
	HardFault_CFSR_DACCVIOL    = SCB_CFSR_DACCVIOL
	HardFault_CFSR_MUNSTKERR   = SCB_CFSR_MUNSTKERR
	HardFault_CFSR_MSTKERR     = SCB_CFSR_MSTKERR
	HardFault_CFSR_MLSPERR     = SCB_CFSR_MLSPERR
	HardFault_CFSR_IBUSERR     = SCB_CFSR_IBUSERR
	HardFault_CFSR_PRECISERR   = SCB_CFSR_PRECISERR
	HardFault_CFSR_IMPRECISERR = SCB_CFSR_IMPRECISERR
	HardFault_CFSR_UNSTKERR    = SCB_CFSR_UNSTKERR
	HardFault_CFSR_STKERR      = SCB_CFSR_STKERR
	HardFault_CFSR_LSPERR      = SCB_CFSR_LSPERR
	HardFault_CFSR_UNDEFINSTR  = SCB_CFSR_UNDEFINSTR
	HardFault_CFSR_INVSTATE    = SCB_CFSR_INVSTATE
	HardFault_CFSR_INVPC       = SCB_CFSR_INVPC
	HardFault_CFSR_NOCP        = SCB_CFSR_NOCP
	HardFault_CFSR_UNALIGNED   = SCB_CFSR_UNALIGNED
	HardFault_CFSR_DIVBYZERO   = SCB_CFSR_DIVBYZERO
	HardFault_CFSR_MMARVALID   = SCB_CFSR_MMARVALID
	HardFault_CFSR_BFARVALID   = SCB_CFSR_BFARVALID
)
