// Copyright (c) 2016-2017 Antony Polukhin
//
// Distributed under the Boost Software License, Version 1.0. (See accompanying
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)

#ifndef BOOST_PFR_DETAIL_CORE14_HPP
#define BOOST_PFR_DETAIL_CORE14_HPP
#pragma once

#include <boost/pfr/detail/config.hpp>

#if BOOST_PFR_USE_LOOPHOLE
#   include <boost/pfr/detail/core14_loophole.hpp>
#else
#   include <boost/pfr/detail/core14_classic.hpp>
#endif

#endif // BOOST_PFR_DETAIL_CORE14_HPP
