.section .flash_config
.global  __flash_config
__flash_config:
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xFF
    .byte 0xDE
    .byte 0xF9
    .byte 0xFF
    .byte 0xFF
