// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// TODO: exclude big endian targets here and include them in endian_big.go, once tinygo supports one

package os

const isBigEndian = false
