/**
 *
 * @defgroup nrfx_systick_config ARM(R) SysTick driver configuration
 * @{
 * @ingroup nrfx_systick
 */
/** @brief 
 *
 *  Set to 1 to activate.
 *
 * @note This is an NRF_CONFIG macro.
 */
#define NRFX_SYSTICK_ENABLED

/** @} */
