// Hand created file. DO NOT DELETE.
// Hardfault aliases for definitions that have inconsistent naming (which are
// auto-generated by gen-device-svd.go) among devices in package nxp.

//go:build nxp && !mimxrt1062
// +build nxp,!mimxrt1062

package nxp

const (
	HardFault_CFSR_IACCVIOL    = SystemControl_CFSR_IACCVIOL
	HardFault_CFSR_DACCVIOL    = SystemControl_CFSR_DACCVIOL
	HardFault_CFSR_MUNSTKERR   = SystemControl_CFSR_MUNSTKERR
	HardFault_CFSR_MSTKERR     = SystemControl_CFSR_MSTKERR
	HardFault_CFSR_MLSPERR     = SystemControl_CFSR_MLSPERR
	HardFault_CFSR_IBUSERR     = SystemControl_CFSR_IBUSERR
	HardFault_CFSR_PRECISERR   = SystemControl_CFSR_PRECISERR
	HardFault_CFSR_IMPRECISERR = SystemControl_CFSR_IMPRECISERR
	HardFault_CFSR_UNSTKERR    = SystemControl_CFSR_UNSTKERR
	HardFault_CFSR_STKERR      = SystemControl_CFSR_STKERR
	HardFault_CFSR_LSPERR      = SystemControl_CFSR_LSPERR
	HardFault_CFSR_UNDEFINSTR  = SystemControl_CFSR_UNDEFINSTR
	HardFault_CFSR_INVSTATE    = SystemControl_CFSR_INVSTATE
	HardFault_CFSR_INVPC       = SystemControl_CFSR_INVPC
	HardFault_CFSR_NOCP        = SystemControl_CFSR_NOCP
	HardFault_CFSR_UNALIGNED   = SystemControl_CFSR_UNALIGNED
	HardFault_CFSR_DIVBYZERO   = SystemControl_CFSR_DIVBYZERO
	HardFault_CFSR_MMARVALID   = SystemControl_CFSR_MMARVALID
	HardFault_CFSR_BFARVALID   = SystemControl_CFSR_BFARVALID
)
